import colight.plot as Plot

#
Plot.Grid(
    ["div.bg-yellow-200"],
    ["div.bg-red-200"],
    ["div.bg-blue-200"],
    ["div.bg-green-200"],
    ["div.bg-yellow-200"],
    ["div.bg-red-200"],
    ["div.bg-blue-200"],
    ["div.bg-green-200"],
)
